/*
	HoverColorBars - Interactive color bars demo
	Written (2016) by Nutchanon Wetchasit <Nutchanon.Wetchasit@gmail.com>
	
	Released to public domain under Creative Commons Zero 1.0
	(https://creativecommons.org/publicdomain/zero/1.0/)
*/
class HoverColorBars {
	private var mc:MovieClip;
	public var width:Number;
	public var height:Number;
	private var pointerX:Number=Number.NaN;
	private var pointerY:Number=Number.NaN;
	private var pointerPressed:Boolean=false;
	
	/* main(): MTASC entry point */
	public static function main(mc:MovieClip):Void {
		var prog:HoverColorBars;
		
		prog=new HoverColorBars(mc);
	}
	
	/* <init>: program constructor */
	public function HoverColorBars(mc:MovieClip) {
		var prog:HoverColorBars;
		
		this.mc=mc;
		prog=this;
		Stage.addListener(this);
		
		mc.onRollOver=function():Void {
			prog.pointerX=this._xmouse;
			prog.pointerY=this._ymouse;
			prog.refresh();
		};
		mc.onMouseMove=function():Void {
			if(this._xmouse<0 || this._xmouse>=prog.width ||
				this._ymouse<0 || this._ymouse>=prog.height) {
				prog.pointerX=Number.NaN;
				prog.pointerY=Number.NaN;
			} else {
				prog.pointerX=this._xmouse;
				prog.pointerY=this._ymouse;
			}
			prog.refresh();
		};
		mc.onRollOut=function():Void {
			prog.pointerX=Number.NaN;
			prog.pointerY=Number.NaN;
			prog.refresh();
		};
		mc.onMouseDown=function():Void {
			prog.pointerPressed=true;
			prog.refresh();
		};
		mc.onMouseUp=function():Void {
			prog.pointerPressed=false;
			prog.refresh();
		};
		
		sizeChanged();
	}
	
	/* refresh(): refresh screen */
	private function refresh():Void {
		if(!isNaN(pointerX) && !isNaN(pointerY)) {
			drawBars(Math.floor(pointerX*8/(width)),pointerPressed?Math.floor(pointerX*8/(width)):-1);
		} else {
			drawBars(-1,-1);
		}
	}
	
	/* drawBars(): color bars drawing function */
	private function drawBars(hovered:Number,clicked:Number):Void {
		mc.clear();
		fillRect(width*0/8,0,width/8,height,contrast(0xFFFFFF,clicked==0?0.67:(hovered==0?1.00:0.85),0));
		fillRect(width*1/8,0,width/8,height,contrast(0xFFFF00,clicked==1?0.67:(hovered==1?1.00:0.85),0));
		fillRect(width*2/8,0,width/8,height,contrast(0x00FFFF,clicked==2?0.67:(hovered==2?1.00:0.85),0));
		fillRect(width*3/8,0,width/8,height,contrast(0x00FF00,clicked==3?0.67:(hovered==3?1.00:0.85),0));
		fillRect(width*4/8,0,width/8,height,contrast(0xFF00FF,clicked==4?0.67:(hovered==4?1.00:0.85),0));
		fillRect(width*5/8,0,width/8,height,contrast(0xFF0000,clicked==5?0.67:(hovered==5?1.00:0.85),0));
		fillRect(width*6/8,0,width/8,height,contrast(0x0000FF,clicked==6?0.67:(hovered==6?1.00:0.85),0));
		fillRect(width*7/8,0,width/8,height,contrast(0x000000,0,clicked==7?0.33:(hovered==7?0.50:0.00)));
	}
	
	/* fillRect(): color-filled rectangle drawing function */
	private function fillRect(x:Number,y:Number,w:Number,h:Number,color:Number):Void {
		mc.endFill();
		mc.lineStyle(undefined);
		mc.moveTo(x,y);
		mc.beginFill(color);
		mc.lineTo(x+w,y);
		mc.lineTo(x+w,y+h);
		mc.lineTo(x,y+h);
		mc.lineTo(x,y);
		mc.endFill();
	}
	
	/* contrast(): set specified color's contrast/brightness offset by factor */
	private function contrast(color:Number,factor:Number,brightFactor:Number):Number {
		var red:Number;
		var green:Number;
		var blue:Number;
		
		if(brightFactor==undefined) brightFactor=0;
		
		red=((color>>16)&0xFF)*factor + (255*brightFactor);
		green=((color>>8)&0xFF)*factor + (255*brightFactor);
		blue=((color)&0xFF)*factor + (255*brightFactor);
		
		if(!(red>=0)) red=0;
		if(!(red<=255)) red=255;
		if(!(green>=0)) green=0;
		if(!(green<=255)) green=255;
		if(!(blue>=0)) blue=0;
		if(!(blue<=255)) blue=255;
		
		return (red<<16)|(green<<8)|(blue);
	}
	
	/* sizeChanged(): stage size change monitoring function */
	private function sizeChanged():Void {
		this.width=Stage.width;
		this.height=Stage.height;
		refresh();
	}
}
